#include "postgres.h"
#include "fmgr.h"
#include "utils/geo_decls.h"

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

PG_FUNCTION_INFO_V1(center_sfunc);

/*
  This function is to create a box over latitude and longitude 
  and sugaggregate to create a box locable in a map.

  https://doxygen.postgresql.org/structBOX.html
  BOX::high / BOX::low (both points)
  https://doxygen.postgresql.org/geo__decls_8h_source.html#l00102

   typedef struct
   {
     Point high,
           low; // corner POINTs 
    }


*/
Datum center_sfunc(PG_FUNCTION_ARGS)
{
    BOX *agg_state = (BOX *)palloc(sizeof(BOX));

    Point *new_point = PG_GETARG_POINT_P(1);

    //double el = PG_GETARG_FLOAT8(1);
    bool isnull = PG_ARGISNULL(0);

    if (isnull)
    {
        //first point
        agg_state->high.x = 0;
        agg_state->high.y = 0;
        agg_state->low.x  = 0;
        agg_state->low.y  = 0;
        //second point 
        PG_RETURN_BOX_P(agg_state);
    }


    agg_state = PG_GETARG_BOX_P(0);

    agg_state->high.x = Max(new_point->x, agg_state->high.x);
    agg_state->high.y = Max(new_point->y, agg_state->high.y);

    agg_state->low.x = Min(new_point->x, agg_state->high.x);
    agg_state->low.y = Min(new_point->y, agg_state->high.y);

    PG_RETURN_BOX_P(agg_state);
}
